@@ -84,6 +84,7 @@ class ForagingEnv(Env):
         force_coop,
         normalize_reward=True,
         grid_observation=False,
+        penalty=0.0,
     ):
         self.logger = logging.getLogger(__name__)
         self.seed()
@@ -91,6 +92,8 @@ class ForagingEnv(Env):
 
         self.field = np.zeros(field_size, np.int32)
 
+        self.penalty = penalty
+        
         self.max_food = max_food
         self._food_spawned = 0.0
         self.max_player_level = max_player_level
@@ -129,10 +132,10 @@ class ForagingEnv(Env):
             max_food = self.max_food
             max_food_level = self.max_player_level * len(self.players)
 
-            min_obs = [-1, -1, 0] * max_food + [0, 0, 1] * len(self.players)
-            max_obs = [field_x, field_y, max_food_level] * max_food + [
-                field_x,
-                field_y,
+            min_obs = [-1, -1, 0] * max_food + [-1, -1, 0] * len(self.players)
+            max_obs = [field_x-1, field_y-1, max_food_level] * max_food + [
+                field_x-1,
+                field_y-1,
                 self.max_player_level,
             ] * len(self.players)
         else:
@@ -454,7 +457,12 @@ class ForagingEnv(Env):
         ndone = [obs.game_over for obs in observations]
         # ninfo = [{'observation': obs} for obs in observations]
         ninfo = {}
-
+        
+        # check the space of obs
+        for i, obs in  enumerate(nobs):
+            assert self.observation_space[i].contains(obs), \
+                f"obs space error: obs: {obs}, obs_space: {self.observation_space[i]}"
+        
         return nobs, nreward, ndone, ninfo
 
     def reset(self):
@@ -540,6 +548,8 @@ class ForagingEnv(Env):
 
             if adj_player_level < food:
                 # failed to load
+                for a in adj_players:
+                    a.reward -= self.penalty
                 continue
 
             # else the food was loaded and each player scores points
